defmodule Nulla.Models.User do
  use Ecto.Schema
  import Ecto.Changeset
  import Ecto.Query
  alias Nulla.Repo
  alias Nulla.Models.User
  alias Nulla.Models.Session

  @primary_key {:id, :integer, autogenerate: false}
  schema "users" do
    field :email, :string
    field :password, :string
    field :privateKeyPem, :string
    field :last_active_at, :utc_datetime

    has_many :user_sessions, Session

    timestamps(type: :utc_datetime)
  end

  @doc false
  def changeset(user, attrs) do
    user
    |> cast(attrs, [
      :id,
      :email,
      :password,
      :privateKeyPem,
      :last_active_at
    ])
    |> validate_required([
      :id,
      :email,
      :password,
      :privateKeyPem,
      :last_active_at
    ])
  end

  def create_user(attrs) when is_map(attrs) do
    %__MODULE__{}
    |> changeset(attrs)
    |> Repo.insert()
  end

  def get_user(by) when is_map(by) or is_list(by) do
    Repo.get_by(User, by)
  end

  def get_total_users_count() do
    Repo.aggregate(from(u in User), :count, :id)
  end

  def get_active_users_count(days) do
    cutoff = DateTime.add(DateTime.utc_now(), -days * 86400, :second)

    from(u in User, where: u.last_active_at > ^cutoff)
    |> Repo.aggregate(:count, :id)
  end

  def update_last_active(user) do
    user
    |> change(last_active_at: DateTime.utc_now())
    |> Repo.update()
  end
end
